@@ -1,7 +1,7 @@
-__version__ = '0.0.29'
+__version__ = '0.0.31'
 
 from .isthmuslib_plots import *
 from .isthmuslib_helpers import *
 from .config import *
-from .sequences import *
+from .vectors import *
 from .utils import *
