@@ -12,10 +12,12 @@ import seaborn as sns
 ## Scatter plot
 ##############################
 
+# LEGACY CODE - NEEDS REFACTOR
 def scatter(xData, yData, xlabel='', ylabel='', title='', xlim=None, ylim=None, figsize=None, facecolor='w',
             xylabelsize=15, titlesize=20, xscale='linear', yscale='linear', markersize=3, markercolor='green',
             grid=False, legend=None, markerstyle='o', lines=False, linestyle='-', linewidth=None,
-            rollingMedianBinWidth=None, rollingMeanBinWidth=None, linecolor=None):
+            rollingMedianBinWidth=None, rollingMeanBinWidth=None, linecolor=None, plotBestFit=False,
+            watermarkText=None, watermarkPlacement=None, watermarkSize=10, watermarkColor='grey'):
     # Process defaults
     if figsize is None:
         figsize = (13, 7)
@@ -56,7 +58,7 @@ def scatter(xData, yData, xlabel='', ylabel='', title='', xlim=None, ylim=None,
                                     temporaryDataFrame['yData'].rolling(rollingMeanBinWidth).mean(),
                                     color=linecolor, linestyle=linestyle, linewidth=linewidth)
                 plt.plot(temporaryDataFrame.xData, temporaryDataFrame['yData'].rolling(rollingMeanBinWidth).mean(),
-                         color=trace.get_facecolor(), linestyle=linestyle, linewidth=linewidth)
+                         color='k', linestyle=linestyle, linewidth=linewidth)
                 thisString = thisKey + ' (rolling mean, bin width = ' + str(rollingMeanBinWidth) + ')'
                 legendHandles.append(trace)
                 legendStrings.append(thisString)
@@ -66,11 +68,21 @@ def scatter(xData, yData, xlabel='', ylabel='', title='', xlim=None, ylim=None,
                                     temporaryDataFrame['yData'].rolling(rollingMedianBinWidth).median(),
                                     color=linecolor, linestyle=linestyle, linewidth=linewidth)
                 plt.plot(temporaryDataFrame.xData, temporaryDataFrame['yData'].rolling(rollingMedianBinWidth).median(),
-                         color=trace.get_facecolor(), linestyle=linestyle, linewidth=linewidth)
+                         color='k', linestyle=linestyle, linewidth=linewidth)
                 thisString = thisKey + ' (rolling median, bin width = ' + str(rollingMedianBinWidth) + ')'
                 legendHandles.append(trace)
                 legendStrings.append(thisString)
 
+        if plotBestFit:
+            m, b = np.polyfit(xData, yDataSet, 1)
+            bestFitY = [m * x + b for x in xData]
+            plt.plot(xData, bestFitY, color='k')
+            # Next few lines disabled until plot handles are handled
+            # trace = plt.plot(xData, bestFitY)
+            # thisString = thisKey + ' (best fit)'
+            # legendHandles.append(trace)
+            # legendStrings.append(thisString)
+
     # Bells and whistles
     plt.xlabel(xlabel, size=xylabelsize)
     plt.ylabel(ylabel, size=xylabelsize)
@@ -85,6 +97,14 @@ def scatter(xData, yData, xlabel='', ylabel='', title='', xlim=None, ylim=None,
         plt.grid(grid)
     if legend:
         plt.legend(legendHandles, legendStrings)
+    if watermarkText is not None:
+        if watermarkPlacement is None:
+            watermarkPlacement = (0.01, 0.97)
+        xLimits = plt.xlim()
+        yLimits = plt.ylim()
+        xCoordinate = xLimits[0] + watermarkPlacement[0] * (xLimits[1] - xLimits[0])
+        yCoordinate = yLimits[0] + watermarkPlacement[1] * (yLimits[1] - yLimits[0])
+        plt.text(xCoordinate, yCoordinate, watermarkText, fontsize=watermarkSize, c=watermarkColor)
     return f
 
 
@@ -92,9 +112,11 @@ def scatter(xData, yData, xlabel='', ylabel='', title='', xlim=None, ylim=None,
 ## Scatter plot (dictionary input)
 ##############################
 
+# LEGACY CODE - NEEDS REFACTOR
 def scatterDictionary(dataDict, xlabel='', ylabel='', title='', xlim=None, ylim=None, figsize=None, facecolor='w',
                       xylabelsize=15, titlesize=20, xscale='linear', yscale='linear', grid=False, legend=None,
-                      lines=False, linestyle='-', linewidth=None):
+                      lines=False, linestyle='-', linewidth=None, watermarkText=None, watermarkPlacement=None,
+                      watermarkSize=10, watermarkColor='grey'):
     # Process defaults
     if figsize is None:
         figsize = (13, 7)
@@ -152,6 +174,14 @@ def scatterDictionary(dataDict, xlabel='', ylabel='', title='', xlim=None, ylim=
         plt.grid(grid)
     if legend:
         plt.legend(legendHandles, legendStrings)
+    if watermarkText is not None:
+        if watermarkPlacement is None:
+            watermarkPlacement = (0.01, 0.97)
+        xLimits = plt.xlim()
+        yLimits = plt.ylim()
+        xCoordinate = xLimits[0] + watermarkPlacement[0] * (xLimits[1] - xLimits[0])
+        yCoordinate = yLimits[0] + watermarkPlacement[1] * (yLimits[1] - yLimits[0])
+        plt.text(xCoordinate, yCoordinate, watermarkText, fontsize=watermarkSize, c=watermarkColor)
     return f
 
 
@@ -159,9 +189,11 @@ def scatterDictionary(dataDict, xlabel='', ylabel='', title='', xlim=None, ylim=
 ## Histogram plot
 ##############################
 
+# LEGACY CODE - NEEDS REFACTOR
 def hist(data, xlabel='', ylabel='frequency', title='', xlim=None, ylim=None, figsize=None, facecolor='w',
          xylabelsize=15, titlesize=20, xscale='linear', yscale='linear', color=None, bins=150, grid=False, legend=None,
-         cumulative=False, density=False, internalFaceColor=None, alpha=None):
+         cumulative=False, density=False, internalFaceColor=None, alpha=None, watermarkText=None,
+         watermarkPlacement=None, watermarkSize=10, watermarkColor='grey'):
     if figsize is None:
         figsize = (13, 7)
 
@@ -211,6 +243,14 @@ def hist(data, xlabel='', ylabel='frequency', title='', xlim=None, ylim=None, fi
         plt.grid(grid)
     if legend:
         plt.legend(legendHandles, legendStrings)
+    if watermarkText is not None:
+        if watermarkPlacement is None:
+            watermarkPlacement = (0.01, 0.97)
+        xLimits = plt.xlim()
+        yLimits = plt.ylim()
+        xCoordinate = xLimits[0] + watermarkPlacement[0] * (xLimits[1] - xLimits[0])
+        yCoordinate = yLimits[0] + watermarkPlacement[1] * (yLimits[1] - yLimits[0])
+        plt.text(xCoordinate, yCoordinate, watermarkText, fontsize=watermarkSize, c=watermarkColor)
     return f
 
 
@@ -220,9 +260,11 @@ def hist(data, xlabel='', ylabel='frequency', title='', xlim=None, ylim=None, fi
 ## Calculates: Z = counts
 ##############################
 
+# LEGACY CODE - NEEDS REFACTOR
 def hist2d(xData, yData, xlabel='', ylabel='frequency', title='', xlim=None, ylim=None, figsize=None,
            facecolor='w', xylabelsize=15, titlesize=20, xscale='linear', yscale='linear', cmap='jet', bins=50,
-           grid=None, range=None, density=False, cmin=None, cmax=None, norm=None, bookends=1):
+           grid=None, range=None, density=False, cmin=None, cmax=None, norm=None, bookends=1, watermarkText=None,
+           watermarkPlacement=None, watermarkSize=10, watermarkColor='grey'):
     # Process defaults
     if figsize is None:
         figsize = (13, 7)
@@ -249,6 +291,14 @@ def hist2d(xData, yData, xlabel='', ylabel='frequency', title='', xlim=None, yli
     plt.yscale(yscale)
     if grid:
         plt.grid(grid)
+    if watermarkText is not None:
+        if watermarkPlacement is None:
+            watermarkPlacement = (0.01, 0.97)
+        xLimits = plt.xlim()
+        yLimits = plt.ylim()
+        xCoordinate = xLimits[0] + watermarkPlacement[0] * (xLimits[1] - xLimits[0])
+        yCoordinate = yLimits[0] + watermarkPlacement[1] * (yLimits[1] - yLimits[0])
+        plt.text(xCoordinate, yCoordinate, watermarkText, fontsize=watermarkSize, c=watermarkColor)
     return f
 
 
@@ -257,10 +307,12 @@ def hist2d(xData, yData, xlabel='', ylabel='frequency', title='', xlim=None, yli
 ## Input: X & Y & Z vectors
 ##############################
 
+# LEGACY CODE - NEEDS REFACTOR
 def heatmapDataFrame(df, xcol='x', ycol='y', zcol='z', xlabel='', ylabel='', annot=False, figsize=None,
                      facecolor='white', title='', xlim=None, ylim=None, vmin=None, vmax=None, xylabelsize=15,
                      titlesize=20, linewidth=None, cmap=None, cbar=True, mask=None, center=None, robust=None,
-                     linecolor=None):
+                     linecolor=None, watermarkText=None, watermarkPlacement=None, watermarkSize=10,
+                     watermarkColor='grey'):
     # Process defaults
     if figsize is None:
         figsize = (13, 7)
@@ -271,12 +323,15 @@ def heatmapDataFrame(df, xcol='x', ycol='y', zcol='z', xlabel='', ylabel='', ann
     return heatmap(x, y, z, annot=annot, xlabel=xlabel, ylabel=ylabel, figsize=figsize,
                    facecolor=facecolor, title=title, xlim=xlim, ylim=ylim, vmin=vmin, vmax=vmax,
                    xylabelsize=xylabelsize, titlesize=titlesize, linewidth=linewidth, cmap=cmap, cbar=cbar, mask=mask,
-                   center=center, robust=robust, linecolor=linecolor)
+                   center=center, robust=robust, linecolor=linecolor, watermarkSize=watermarkSize,
+                   watermarkText=watermarkText, watermarkPlacement=watermarkPlacement)
 
 
+# LEGACY CODE - NEEDS REFACTOR
 def heatmap(x, y, z, xlabel='x', ylabel='y', annot=False, figsize=None, facecolor='white', title='',
             xlim=None, ylim=None, vmin=None, vmax=None, xylabelsize=15, titlesize=20, linewidth=None, cmap=None,
-            cbar=True, mask=None, center=None, robust=None, linecolor=None):
+            cbar=True, mask=None, center=None, robust=None, linecolor=None, watermarkText=None,
+            watermarkPlacement=None, watermarkSize=10, watermarkColor='grey'):
     # Process defaults
     if figsize is None:
         figsize = (13, 7)
@@ -301,6 +356,14 @@ def heatmap(x, y, z, xlabel='x', ylabel='y', annot=False, figsize=None, facecolo
         plt.xlim(xlim)
     if ylim:
         plt.ylim(ylim)
+    if watermarkText is not None:
+        if watermarkPlacement is None:
+            watermarkPlacement = (0.01, 0.97)
+        xLimits = plt.xlim()
+        yLimits = plt.ylim()
+        xCoordinate = xLimits[0] + watermarkPlacement[0] * (xLimits[1] - xLimits[0])
+        yCoordinate = yLimits[0] + watermarkPlacement[1] * (yLimits[1] - yLimits[0])
+        plt.text(xCoordinate, yCoordinate, watermarkText, fontsize=watermarkSize, c=watermarkColor)
     return f
 
 
@@ -309,6 +372,7 @@ def heatmap(x, y, z, xlabel='x', ylabel='y', annot=False, figsize=None, facecolo
 ##############################
 # To do - log x-axis might not work
 
+# LEGACY CODE - NEEDS REFACTOR
 def shotgunPlots1d(df, useFields=None, xscale='linear', yscale='linear', figsize=(13, 7), cumulative=False,
                    density=False):
     if useFields == None:
@@ -325,6 +389,7 @@ def shotgunPlots1d(df, useFields=None, xscale='linear', yscale='linear', figsize
             pass
 
 
+# LEGACY CODE - NEEDS REFACTOR
 def shotgunPlots2d(df, xColName, useFields=None, xscale='linear', yscale='linear', figsize=(13, 7),
                    types=['scatter', 'hist2d']):
     if useFields == None:
